import bleach
from django.contrib import messages
from django.contrib.auth.decorators import login_required, user_passes_test
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect, render

from tickets.forms import TicketForm
from tickets.models import Task, TaskList
from tickets.utils import send_notify_mail, staff_check


@login_required
@user_passes_test(staff_check)
def list_detail(request, list_id=None, list_slug=None) -> HttpResponse:
    if list_slug == "mine":
        tickets = Task.objects.filter(assigned_to=request.user).select_related("created_by", "assigned_to")
        context = {
                "tickets": tickets,
                "list_id": list_id,
                "list_slug": list_slug
            }
    else:
        task_list = get_object_or_404(TaskList, id=list_id)
        if task_list.group not in request.user.groups.all() and not request.user.is_superuser:
            raise PermissionDenied
        tickets = Task.objects.filter(task_list=task_list.id).select_related("created_by", "assigned_to")

        form = TicketForm(request.user, request.POST, initial={"task_list": task_list})
        if request.POST.getlist("ticket_create"):
            if form.is_valid():
                new_ticket = form.save(commit=False)
                new_ticket.created_by = request.user
                new_ticket.note = bleach.clean(form.cleaned_data["note"], strip=True)
                new_ticket.task_list = task_list
                new_ticket.status = new_ticket.first_state()
                form.save()
                
                if (
                    "notify" in request.POST
                    and new_ticket.assigned_to
                    and new_ticket.assigned_to != request.user
                ):
                    send_notify_mail(new_ticket)

                messages.success(request, 'New ticket "{t}" has been added.'.format(t=new_ticket.title))
                return redirect(request.path)

        context = {
            "form": form,
            "list_id": list_id,
            "list_slug": list_slug,
            "task_list": task_list,
            "tickets": tickets,
        }

    return render(request, "tickets/list_detail.html", context)
